;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Demographic microsimulation model modified to read in Liam2 data as in simple2001.h5
; 
; 
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

extensions [profiler]

globals [
  pop years hsize houses myage currentcase currenthouse maxhouseholds
  pcsingle pcmarried pccohab pcseparated pcdivorced pcwidowed upper lower
  currentagent
  school_age working_age retired current_year
] 

patches-own [
  pcase               ; patch address
  period              ; year
  ]

turtles-own [
  case           ; case number
  id
  partnerid      ; who number of partner
  parentid       ; who number of parent
  person         ; household member number
  homex          ; x coordinate of original address
  homey          ; y coordinate of original address
  age            ; years
  under20        ; aged under 20
  twenties       ; aged in twenties
  thirties       ; aged in thirties
  forties        ; aged in forties
  fifties        ; aged in fifties
  sixties        ; aged 60 and over  
  sex            ; male female
  male           ; sex read in from file
  nsex           ; numerical sex read from file
  nstatus        ; numerical marrital status read from file
  civilstate     ; marital status read from file
  m_id           ; mother's id
  partner_id     ; id of partner
  hh_id          ; id of household
  per              ; period
  status         ; married cohabiting single widowed divorced separated samesex
  previousstatus ; previous marital status
  newwidow       ; widowed in current mortality module
  newlyseparated ; 0 = no: number indicates time since separation in years
  children       ; number of children born by agent
  numdependents  ; number of agents responsible for
  newbirth       ; birth in current fertility module
  dur_in_couple  ; years married
  numinhouse     ; number of individuals in the household
  student        ; currently a full time student
  education      ; age completed full time education
  changed        ; recent cohabitation dissolution indicator
  newcohabit     ; cohabitation began in current year
  nemployment    ; numerical employment status from file
  employment     ; type of employment
  workstate      ; employment status read from file
]

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; initialisation
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

to setup
  clear-all
  set-default-shape turtles "house"
  init-patches
  initialise
  set years 0
  set current_year 2001

  carefully [file-delete "outputmeans.dat"] []
  carefully [file-delete "outputpc.dat"] []    
  set propensity_to_marry 1
  set propensity_to_divorce 1
  set propensity_to_cohabit 1
  set propensity_to_split 1
  set birth_rate 1
  set mortality_rate 1
  summarise
  update-plot
end

to init-patches        ; read in household data
  let numhouses 0      ; house counter
  let lasthouse -1      ; highest house so far
  set currenthouse 0   ; number of current household
  ask patches [set pcase -1]
  file-open "onetenh.txt"
  show "reading household file"
  if households > 0 [
    while [not file-at-end?] [
      set currenthouse file-read
        if currenthouse > lasthouse [
        set numhouses numhouses + 1
        if numhouses > households  [file-close stop]
        let x int (currenthouse / (max-pycor + 1))
        let y remainder currenthouse (max-pycor + 1)
        ask patch x y [
          set pcase currenthouse
          set period file-read
          set pcolor green
        ]
        set lasthouse currenthouse
      ]  
    ]  
  ]
  file-close
end

to initialise  ; set up initial agent population
  let r 0.0
  let recnum 0
  let finished 0
  set maxhouseholds max [pcase] of patches
  file-open "onetenp.txt"
  show "reading agent file"
  while [not file-at-end? and finished = 0] [
    set recnum file-read
    if-else recnum <= maxhouseholds [
      cro 1 [                  ; create a new agent
        set case recnum        ; set turtle identifier from file
        set currentcase case   ; set current case number
        set id file-read
        set age file-read
        set male file-read
        set workstate file-read
        set civilstate file-read
        set dur_in_couple file-read
        set m_id file-read
        set partner_id file-read
        set per file-read          ; period from file

        setagebands  ; initialise or update age bands according to current age

        if male = true [set sex "male" set color blue]
        if male = false [set sex "female" set color pink]

        if civilstate = 1 [set status "single"]
        if civilstate = 2 [set status "married"]
        if civilstate = 3 [set status "divorced"]
        if nstatus = 4 [set status "widowed"]
        
        set previousstatus "single"
        if status = "separated" [set previousstatus "married"]
        if status = "widowed" [set previousstatus "married"]
        if status = "divorced" [set previousstatus "separated"]
        
        set student 0
        if age <= 16 [set student 1 set employment "Full Time Student" set nemployment 6]
        
        if workstate <= 0 [set employment "Unknown"]
        if workstate = 1 [set employment "In Work"]
        if workstate = 2 [set employment "Unemployed"]
        if workstate = 3 [set employment "In Education" set student 1]
        if workstate = 4 [set employment "Retired"]
        if workstate = 5 [set employment "Other"]
        if workstate = 6 [set employment "Other Inactive"]
        
        let x int (currentcase / (max-pycor + 1))
        let y remainder currentcase (max-pycor + 1)
        move-to patch x y  ; go to the family address
        set homex pxcor          ; initialise original address
        set homey pycor          ; initialise original address
      ]
    ]    
    [set finished 1]
  ]
  file-close
  show "finished reading agent file"
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;control routines
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

to go                  ; main loop
  profiler:start
  if years = stop_after [
  profiler:stop
  output-print profiler:report
  profiler:reset  
    stop
  ]
  ask turtles [updatedemographics]
  followparent
  clear_patches
  summarise
  set current_year current_year + 1   
  writetofile
  update-plot
  set years years + 1  ; increment calender
end

to clear_patches
  ask patches with [count turtles-here = 0] [clear_patch]
end

to clear_patch
  set pcase -1
  set period 0
  set pcolor black
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; demographic section
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

to updatedemographics  ; demographic modules
  let r 0.0
  set r random-float 1
  set currentagent who
  
  set newlyseparated 0  ; reset new separation indicator
  set changed 0         ; reset newly out of cohabitation indicator
  set newcohabit 0      ; reset new cohabitation indicator
  set newwidow 0        ; reset new widow indicator
  set newbirth 0        ; reset new birth indicator
   
  terminate           ; mortality
  create              ; fertility
  if student = 1 [educate]             ; education module
  if sex = "male" and age >= 65 [set employment "Retired" set nemployment 4]
  if sex = "female" and age >= 60 [set employment "Retired" set nemployment 4]
  leavehome           ; leave parental home

  ; partnership dissolution modules
  set r random-float 1  ; decide whether male or female dominated this time through
  if r < 0.5 [  
    if sex = "male" and age >= 16 and age <= 69 and status = "married" [marriagedism]
  ]
  if r >= 0.5 [
    if sex = "female" and age >= 16 and age <= 69 and status = "married" [marriagedisf] 
  ]
  if r < 0.5 [
    if sex = "male" and age >= 16 and age <= 69 and status = "cohabiting" [cohabdism]  
  ]
  if r >= 0.5 [  
    if sex = "female" and age >= 16 and age <= 69 and status = "cohabiting" [cohabdisf]
  ]

  set myage age         ; marriage and cohabitation modules

  if r < 0.5 [
    if sex = "male" and age >= 16 and age <= 69 and newwidow = 0 and status != "married" and status != "cohabiting" [cohabitm]  
  ]
  if r >= 0.5 [  
    if sex = "female" and age >= 16 and age <= 69 and newwidow = 0 and status != "married" and status != "cohabiting" [cohabitf]
  ]

  if r < 0.5 [
    if sex = "male" and age >= 16 and age <= 69 and status = "cohabiting" and newcohabit = 0 [marrycohabm]
    if sex = "male" and age >= 16 and age <= 69 and newwidow = 0
      and status != "married" and newlyseparated != 1 and changed = 0 and status != "cohabiting" [marrym]
  ]
  if r >= 0.5 [
    if sex = "female" and age >= 16 and age <= 69 and status = "cohabiting" and newcohabit = 0 [marrycohabf]
    if sex = "female" and age >= 16 and age <= 69 and newwidow = 0
      and status != "married" and newlyseparated != 1 and changed = 0 and status != "cohabiting" [marryf] 
  ]

  set age age + 1       ; incerment age ready for next cycle
  setagebands           ; update age groups
  if age >= 65 [set employment "Retired" set nemployment 4]
  
  set r random-float 1
  if status = "separated" and r < 0.193 [set status "divorced" set previousstatus "divorced"]
  set r random-float 1
  if status = "separated" and r < 0.025 [set status "single" set previousstatus "divorced"]
end

to terminate ; mortality module
  let r 0.0  ; random number 0 to 1 uniform distribution
  let p 0.0  ; probability of death in current year
  set r random-float 1

  if sex = "male" and age <= 40 [set p 0.000736]
  if sex = "male" and age > 40 [set p 0.00003 * (e ^ (0.0954 * age))] 
  
  if sex = "female" and age <= 40 [set p 0.000411]
  if sex ="female" and age > 40 [set p 0.00001 * (e ^ (0.1017 * age))]
  if p > 0.5 [set p 0.5]
  set p p * mortality_rate
  if r < p [
    if sex = "male" [
      ask turtles-here with [sex = "female" and status = "married"] 
        [set previousstatus "married" set status "widowed" set newwidow 1 set color brown] 
    ]
    if sex = "female" [
      ask turtles-here with [sex = "male" and status = "married"] 
        [set previousstatus "married" set status "widowed" set newwidow 1 set color gray] 
    ]      
  die
  ] 
end

to educate  ; education module
  let r 0.0
  if age = 16 and r < 0.234 [set student 0 set education age set employment "Employed" set nemployment 2]
  if age = 17 and r < 0.3 [set student 0 set education age set employment "Employed" set nemployment 2]
  if age = 18 and r < 0.297 [set student 0 set education age set employment "Employed" set nemployment 2]
  if age = 19 and r < 0.3 [ set student 0 set education age set employment "Employed" set nemployment 2]
  if age = 20 and r < 0.193 [set student 0 set education age set employment "Employed" set nemployment 2]
  if age = 21 and r < 0.028[ set student 0 set education age set employment "Employed" set nemployment 2]
  if age >= 22 [set student 0 set education age set employment "Employed" set nemployment 2] 
end

to create  ; fertility module
  let r 0.0
  let youngest 0   ; age of the youndest child
  let logit 0.0    ; logistic regression based on age, number of childrean and age of youngest child
  let p 0.0
  set r random-float 1
  if sex = "female" and age >= 15 and age <= 60 [
    set youngest min [age] of turtles-here
    
    if status = "married" [
      set logit 2.875 + (-0.155 * age) + (-0.264 * children) + (-0.002 * youngest)
      set p 1 / (1 + e ^ (-(logit)))
      set p p * birth_rate
      
      if r < p [
        set color yellow
        birth     ; generate one offspring
        mb        ; test for multiple births
      ]  
    ]
    if status = "cohabiting" [
      set logit -0.44 + (-0.078 * age) + (0.168 * children) + (0.01 * youngest)
      set p 1 / (1 + e ^ (-(logit)))
      set p p * birth_rate
      
      if r < p [
        set color yellow
        birth 
        mb
      ]  
    ]    
    if status = "single" or status = "divorced" or status = "widowed" or status = "separated" [
      set logit -3.192 + (-0.031 * age) + (0.466 * children) + (-0.021 * youngest)
      set p 1 / (1 + e ^ (-(logit)))
      set p p * birth_rate
      
      if r < p [
        set color yellow
        set r random-float 1
        if r < 0.8 and count turtles-here > 1 [move-to one-of patches with [not any? turtles-here]]
        birth
        mb
      ]  
    ]    
  ]  
end

to birth                 ; generate a new agent
  let r 0.0
  set r random-float 1
  set children children + 1
  set newbirth 1
  hatch 1 [
    set age 0
    setagebands
    set status "single"
    set previousstatus "single"
    set parentid currentagent
    set children 0
    set student 1
    set education 0
    set employment "Student"
    set nemployment 6
    set heading heading + 23
    set r random-float 1
    if r < 0.488 [set sex "female" set color pink]
    if r >= 0.488 [set sex "male" set color blue]
    set color white
    set homex pxcor
    set homey pycor
  ]
end

to mb  ; probability of a multiple birth
  let r 0.0
  set r random-float 1
  if age <= 19 and r < 0.00626 [birth]    ; test for twins
  if age >= 20 and age <= 24 and r < 0.00861 [birth]
  if age >= 25 and age <= 29 and r < 0.01177 [birth]
  if age >= 30 and age <= 34 and r < 0.01510 [birth]
  if age >= 35 and r < 0.01767 [birth]

  if age >= 20 and age <= 24 and r < 0.00009 [birth]  ; triplets
  if age >= 25 and age <= 29 and r < 0.00024 [birth]
  if age >= 30 and age <= 34 and r < 0.00068 [birth]
  if age >= 35 and r < 0.00085 [birth]
end

to leavehome  ; leave parental home
  let r 0.0
  set r random-float 1
  countdependents
  if count turtles-here > 1 and age >= 18 and status = "single" and numdependents = 0 and r < 0.1 [
    move-to one-of patches with [not any? turtles-here]
  ]
  if count turtles-here > 1 and age >= 25 and status = "single" and numdependents = 0 and r < 0.2 [
    move-to one-of patches with [not any? turtles-here]
  ]
  if count turtles-here > 1 and age >= 19 and status = "single" and education = 1 and  numdependents = 0 and r < 0.7 [
    move-to one-of patches with [not any? turtles-here]    ; university students living away from parental home
  ]  
end

to countdependents
  let tempid 0
  set tempid who
  set numdependents count turtles-here with [parentid = tempid]
end

to marrycohabm
  let r 0.0
  let p 0.0
  let logit 0.0
  set r random-float 1
    
  set logit -1.807 + (0.015 * thirties) + (-0.541 * forties) + (-1.001 * fifties) + (-0.829 * sixties)                          
  set p 1 / (1 + e ^ (-(logit)))      ; probability of marrying cohabitee                                                 
  set p p * propensity_to_marry
  if r < p [
    if any? turtles-here with [        ; check female is present
      sex = "female"
      and status = "cohabiting"
      and changed = 0
    ] [
      set color green
      set previousstatus status
      set status "married"
      ask one-of turtles-here with [
        sex = "female" 
        and status = "cohabiting"
        and changed = 0
      ] [
        set color green
        set previousstatus status
        set status "married"
      ]
    ]   
  ]
end

to marrycohabf
  let r 0.0
  let p 0.0
  let logit 0.0
  set r random-float 1
    
  set logit -1.862 + (-0.126 * thirties) + (-0.121 * forties) + (-0.708 * fifties) + (-1.113 * sixties)
  set p 1 / (1 + e ^ (-(logit)))      ; probability of marrying cohabitee
  set p p * propensity_to_marry
  
  if r < p [
    if any? turtles-here with [        ; check female is present
      sex = "male"
      and status = "cohabiting"
      and changed = 0
    ] [
      set color green
      set previousstatus status
      set status "married"
      ask one-of turtles-here with [
        sex = "male" 
        and status = "cohabiting"
        and changed = 0
      ] [
        set color green
        set previousstatus status
        set status "married"
      ]
    ]   
  ]
end

to marrym
  let p 0.0        ; probability of marriage for males
  let t 0          ; turtle number
  let r 0.0        ; random number
  let w 0          ; widowed
  let d 0          ; divorced
  let s 0          ; separated
  let logit 0.0    ; logistic regression based on age and marital status
  let found 0      ; located partner on this pass
  
  if status = "widowed" [set w 1]
  if status = "divorced" [set d 1]
  if status = "separated" [set s 1]
  
  set r random-float 1
  if not any? turtles with [
    sex = "female"              ; check if there is anyone available
    and status != "married"
    and status != "cohabiting"
    and newlyseparated != 1
    and age >= 16
    and changed = 0          ; did not separate from cohabitation in the current years dissolution module
    and age > (myage - 15)
    and age < (myage + 15)] [
      show "No agent found" stop
  ]
    
  set logit -4.207 + (-0.001 * age) + (0.137 * w) + (0.692 * d) + (1.684 * s)
  set p 1 / (1 + e ^ (-(logit)))                                                      
  set p p * propensity_to_marry
  
  if r < p [
    setlimitsm
    set previousstatus status 
    set status "married"
    set color violet
    set t who
    move-to one-of patches with [not any? turtles-here]
      
    if any? turtles with [          ; seek male of suitable age
      sex = "female"
      and status != "married"
      and status != "cohabiting"
      and newlyseparated != 1
      and age >= 16
      and changed = 0
      and age >= (myage + lower)
      and age <= (myage + upper)
    ] [  
    set found 1
      ask one-of turtles with [
        sex = "female" 
        and status != "married"
        and status != "cohabiting"
        and newlyseparated != 1
        and age >= 16
        and changed = 0
        and age >= (myage + lower)
        and age <= (myage + upper)
      ] [
        move-to turtle t
        set color green
        set previousstatus status
        set status "married"
      ]   
    ]   
      
    if found = 0 [              ; widen age range
      ask one-of turtles with [
        sex = "female" 
        and status != "married"
        and status != "cohabiting"
        and newlyseparated != 1
        and age >= 16
        and changed = 0
        and age > (myage - 15)
        and age < (myage + 15)
      ] [
        move-to turtle t
        set color green
        set previousstatus status
        set status "married"
      ]
    ]
  ]
end

to marryf
  let p 0.0        ; probability of marriage for females
  let t 0          ; turtle number
  let r 0.0        ; random number
  let w 0          ; widowed
  let d 0          ; divorced
  let s 0          ; separated
  let logit 0.0    ; logistic regression based on age and marital status
  let found 0      ; located partner on this pass
  
  if status = "widowed" [set w 1]
  if status = "divorced" [set d 1]
  if status = "separated" [set s 1]
  
  set r random-float 1
  if not any? turtles with [
    sex = "male"              ; check if there is anyone available
    and status != "married"
    and status != "cohabiting"
    and newlyseparated != 1
    and age >= 16
    and changed = 0
    and age > (myage - 15)
    and age < (myage + 15)] [
      show "No agent found" stop
  ]
    
  set logit -3.408 + (-0.028 * age) + (0.172 * w) + (0.686 * d) + (2.1 * s) + (0.644 * newbirth)                          
  set p 1 / (1 + e ^ (-(logit)))                                                      
  set p p * propensity_to_marry
               
  if r < p [
    setlimitsf
    set previousstatus status     
    set status "married"
    set color violet
    set t who
    move-to one-of patches with [not any? turtles-here]
      
    if any? turtles with [          ; seek male of suitable age
      sex = "male"
      and status != "married"
      and status != "cohabiting"
      and newlyseparated != 1
      and age >= 16
      and changed = 0
      and age >= (myage + lower)
      and age <= (myage + upper)
    ] [  
    set found 1
    ask one-of turtles with [
      sex = "male" 
      and status != "married"
      and status != "cohabiting"
      and newlyseparated != 1
      and age >= 16
      and changed = 0
      and age >= (myage + lower)
      and age <= (myage + upper)
    ] [
      move-to turtle t
      set color green
      set previousstatus status
      set status "married"
      ]   
    ]   
      
    if found = 0 [              ; widen age range
      ask one-of turtles with [
        sex = "male" 
        and status != "married"
        and status != "cohabiting"
        and newlyseparated != 1
        and age >= 16
        and changed = 0
        and age > (myage - 15)
        and age < (myage + 15)
        ] [
        move-to turtle t
        set color green
        set previousstatus status        
        set status "married"
      ]
    ]
  ]
end

to cohabitm
  let p 0.0        ; probability of cohabitation for males
  let t 0          ; turtle number
  let r 0.0        ; random number
  let w 0          ; widowed
  let d 0          ; divorced
  let s 0          ; separated
  let logit 0.0    ; logistic regression based on age and marital status
  let found 0      ; located partner on this pass
  
  if status = "widowed" [set w 1]
  if status = "divorced" [set d 1]
  if status = "separated" [set s 1]
  
  set r random-float 1
  if not any? turtles with [
    sex = "female"              ; check if there is anyone available
    and status != "married"
    and status != "cohabiting"
    and age >= 16
    and age > (myage - 20)
    and age < (myage + 20)] [
      show "No agent found" stop
  ]
    
  set logit -2.199 + (-0.021 * age) + (-0.033 * w) + (0.67 * d) + (1.21 * s)                          
  set p 1 / (1 + e ^ (-(logit)))                                                      
  set p p * propensity_to_cohabit
  
  if r < p [
    setlimitsm
    set previousstatus status  ; store old marital status
    set status "cohabiting"
    set newcohabit 1
    set color violet
    set t who
    move-to one-of patches with [not any? turtles-here]
      
    if any? turtles with [          ; seek female of suitable age
      sex = "female"
      and status != "married"
      and status != "cohabiting"
      and age >= 16
      and age >= (myage + lower)
      and age <= (myage + upper)
    ] [  
    set found 1
      ask one-of turtles with [
        sex = "female" 
        and status != "married"
        and status != "cohabiting"
        and age >= 16
        and age >= (myage + lower)
        and age <= (myage + upper)
      ] [
      move-to turtle t
      set color green
      set previousstatus status      
      set status "cohabiting"
      set newcohabit 1
      ]   
    ]   
      
    if found = 0 [              ; widen age range
      ask one-of turtles with [
        sex = "female" 
        and status != "married"
        and status != "cohabiting"
        and age >= 16
        and age > (myage - 20)
        and age < (myage + 20)
        ] [
        move-to turtle t
        set color green
        set previousstatus status    
        set status "cohabiting"
        set newcohabit 1
      ]
    ]
  ]
end

to cohabitf
  let p 0.0        ; probability of cohabiting for females
  let t 0          ; turtle number
  let r 0.0        ; random number
  let w 0          ; widowed
  let d 0          ; divorced
  let s 0          ; separated
  let logit 0.0    ; logistic regression based on age, marital status and birth from fertility module
  let found 0      ; located partner on this pass
  
  if status = "widowed" [set w 1]
  if status = "divorced" [set d 1]
  if status = "separated" [set s 1]
  
  set r random-float 1
  if not any? turtles with [
    sex = "male"              ; check if there is anyone available
    and status != "married"
    and status != "cohabiting"
    and age >= 16
    and age > (myage - 20)
    and age < (myage + 20)] [
      show "No agent found" stop
  ]
    
  set logit -1.357 + (-0.045 * age) + (-0.69 * w) + (0.536 * d) + (0.478 * s) + (0.757 * newbirth)                          
  set p 1 / (1 + e ^ (-(logit)))                                                      
  set p p * propensity_to_cohabit
                 
  if r < p [
    setlimitsf
    set previousstatus status   
    set status "cohabiting"
    set newcohabit 1
    set color violet
    set t who
    move-to one-of patches with [not any? turtles-here]
      
    if any? turtles with [          ; seek male of suitable age
      sex = "male"
      and status != "married"
      and status != "cohabiting"
      and age >= 16
      and age >= (myage + lower)
      and age <= (myage + upper)
    ] [  
    set found 1
      ask one-of turtles with [
        sex = "male" 
        and status != "married"
        and status != "cohabiting"
        and age >= 16
        and age >= (myage + lower)
        and age <= (myage + upper)
      ] [
      move-to turtle t
      set color green
      set previousstatus status     
      set status "cohabiting"
      set newcohabit 1
      ]   
    ]   
      
    if found = 0 [              ; widen age range
      ask one-of turtles with [
        sex = "male" 
        and status != "married"
        and status != "cohabiting"
        and age >= 16
        and age > (myage - 20)
        and age < (myage + 20)
        ] [
        move-to turtle t
        set color green
        set previousstatus status
        set status "cohabiting"
        set newcohabit 1
      ]
    ]
  ]
end

to marriagedism
  let p 0.0  ; probability of separation from marriage for males
  let r 0.0
  let x 0
  let y 0
  let logit 0.0 ; logistic regression model depending on ageband
  set r random-float 1
  
  set logit -3.112 + (-0.627 * thirties) + (-0.954 * forties) + (-1.848 * fifties) + (-3.078 * sixties)
  set p 1 / (1 + e ^ (-(logit)))                                                      
  set p p * propensity_to_divorce
  
  if r < p [
    if any? turtles-here with [sex = "female" and status = "married"] [
      set r random-float 1        ; decide which agent moves out 
      ask one-of turtles-here with [sex = "female" and status = "married"] [
        set color red
        set status "separated" ; change leavers marital status to separated
        set previousstatus "married"
        set newlyseparated 1
        if r <= 0.1 [
          move-to one-of patches with [not any? turtles-here]       ; female moves out
        ]
      ]
      set status "separated"   ; current agent marital status set to separated
      set previousstatus "married"
      set newlyseparated 1
      set color orange
      if r > 0.1 [
        move-to one-of patches with [not any? turtles-here]          ; male moves out
      ]   
    ]
  ] 
end

to marriagedisf
  let p 0.0  ; probability of separation from marriage for females
  let r 0.0
  let x 0
  let y 0
  let logit 0.0 ; logistic regression model depending on ageband and birth from the fertility module
  set r random-float 1
  
  set logit -2.947 + (-1.134 * newbirth) + (-0.503 * thirties) + (-1.036 * forties) + (-1.938 * fifties) + (-3.378 * sixties)                          
  set p 1 / (1 + e ^ (-(logit)))                                                      
  set p p * propensity_to_divorce
  
  if r < p [
    if any? turtles-here with [sex = "male" and status = "married"] [
      set r random-float 1        ; decide which agent moves out
      ask one-of turtles-here with [sex = "male" and status = "married"] [
        set color red
        set status "separated" ; change leavers marital status divorced
        set previousstatus "married"
        set newlyseparated 1
        if r < 0.9 [
          move-to one-of patches with [not any? turtles-here]    ; male moves out
        ]
      ]
      set status "separated"   ; current agent marital status set to divorced
      set previousstatus "married"
      set newlyseparated 1
      set color orange
      if r >= 0.9 [   
        move-to one-of patches with [not any? turtles-here]      ; female moves out
      ]   
    ]
  ] 
end

to cohabdism
  let p 0.0  ; probability of separation from cohabitation for males
  let r 0.0
  let x 0
  let y 0
  let logit 0.0 ; logistic regression model depending on ageband
  set r random-float 1
  
  set logit -1.932 + (-0.671 * thirties) + (-1.438 * forties) + (-0.611 * fifties) + (-2.064 * sixties)                          
  set p 1 / (1 + e ^ (-(logit)))                                                      
  set p p * propensity_to_split
  
  if r < p [
    if any? turtles-here with [sex = "female" and status = "cohabiting"] [
      set r random-float 1        ; decide which agent moves out
      ask one-of turtles-here with [sex = "female" and status = "cohabiting"] [
        set color red
        set status previousstatus ; change leavers marital status to what it was before
        set previousstatus "cohabiting"
        set changed 1
        if r <= 0.1 [  
          move-to one-of patches with [not any? turtles-here]    ; female moves out  
        ]
      ]
      set status previousstatus   ; current agent reverts to previous marital status
      set previousstatus "cohabiting"
      set changed 1
      set color orange
      if r > 0.1 [
        move-to one-of patches with [not any? turtles-here]     ; male moves out
      ]   
    ]
  ] 
end

to cohabdisf
  let p 0.0  ; probability of separation from cohabitation for females
  let r 0.0
  let x 0
  let y 0
  let logit 0.0 ; logistic regression model depending on ageband and birth from the fertility module
  set r random-float 1
  
  set logit -1.923 + (-0.526 * newbirth) + (-0.523 * thirties) + (-0.898 * forties) + (-1.115 * fifties) + (-1.197 * sixties)                          
  set p 1 / (1 + e ^ (-(logit)))                                                     
  set p p * propensity_to_split
  
  if r < p [
    if any? turtles-here with [sex = "male" and status = "cohabiting"] [
      set r random-float 1        ; decide which agent moves out
      ask one-of turtles-here with [sex = "male" and status = "cohabiting"] [
        set color red
        set status previousstatus ; change leavers marital status to what it was before
        set previousstatus "cohabiting"
        set changed 1
        if r < 0.9 [
          move-to one-of patches with [not any? turtles-here]     ; male moves out       
        ]
      ]
      set status previousstatus   ; current agent reverts to previous marital status
      set previousstatus "cohabiting"
      set changed 1
      set color orange
      if r >= 0.9 [
        move-to one-of patches with [not any? turtles-here]      ; female moves out
      ]   
    ]
  ] 
end

to followparent
  let tempid 0
  let r 0.0
  set r random-float 1
  if r < 0.5 [
    ask turtles with [age <= 25 and status != "married" and status != "cohabiting" and student = 0 and parentid > 0] [
      set tempid parentid
      if any? turtles with [who = tempid] [move-to turtle parentid]
    ]  
  ]  
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; look up tables
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

to setlimitsf
; age differences of new partnerships
; lower is the youngest person they will partner, upper is the oldest
  let r 0.0
  set r random-float 1
  if r >= 0 and r < 0.11 [set lower 10 set upper 15]
  if r >= 0.11 and r < 0.31 [set lower 5 set upper 9]
  if r >= 0.31 and r < 0.46 [set lower 3 set upper 4]
  if r >= 0.46 and r < 0.65 [set lower 1 set upper 2]
  if r >= 0.65 and r < 0.74 [set lower 0 set upper 0]
  if r >= 0.74 and r < 0.85 [set upper -1 set lower -2]
  if r >= 0.85 and r < 0.91 [set upper -3 set lower -4]
  if r >= 0.91 and r <= 1 [set upper -5 set lower -10]
end

to setlimitsm
  let r 0.0
  set r random-float 1
  if r >= 0 and r < 0.11 [set lower -15 set upper -10]
  if r >= 0.11 and r < 0.31 [set lower -9 set upper -5]
  if r >= 0.31 and r < 0.46 [set lower -4 set upper -3]
  if r >= 0.46 and r < 0.65 [set lower -2 set upper -1]
  if r >= 0.65 and r < 0.74 [set lower 0 set upper 0]
  if r >= 0.74 and r < 0.85 [set lower 1 set upper 2]
  if r >= 0.85 and r < 0.91 [set lower 3 set upper 4]
  if r >= 0.91 and r <= 1 [set lower 5 set upper 10]
end  

to setagebands
  set under20 0 set twenties 0 set thirties 0
  set forties 0 set fifties 0 set sixties 0
        
  if age < 20 [set under20 1]
  if age >= 20 and age < 30 [set twenties 1]
  if age >= 30 and age < 40 [set thirties 1]
  if age >= 40 and age < 50 [set forties 1]
  if age >= 50 and age < 60 [set fifties 1]
  if age >= 60 [set sixties 1]
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; output preparation
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

to summarise                    ; set up global variables after annual cycle
  set pop count turtles
  if pop = 0 [stop]
  set houses count patches with [count turtles-here > 0]
  set hsize (pop / houses)
  set pcsingle count turtles with [status = "single"]
  set pcsingle (pcsingle / pop) * 100
  set pccohab count turtles with [status = "cohabiting"]
  set pccohab (pccohab / pop) * 100
  set pcmarried count turtles with [status = "married"]
  set pcmarried (pcmarried / pop) * 100
  set pcseparated count turtles with [status = "separated"]
  set pcseparated (pcseparated / pop) * 100
  set pcdivorced count turtles with [status = "divorced"]
  set pcdivorced (pcdivorced / pop) * 100  
  set pcwidowed count turtles with [status = "widowed"]
  set pcwidowed (pcwidowed / pop) * 100
  
  set school_age count turtles with [age < 18]
  set school_age (school_age / pop) * 100
  set working_age count turtles with [age >= 18 and age < 65]
  set working_age (working_age / pop) * 100
  set retired count turtles with [age >= 65]
  set retired (retired / pop) * 100
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; file output section
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

to writetofile
  file-open "outputpc.dat"
  file-print current_year
  file-print pcsingle
  file-print pcmarried
  file-print pccohab
  file-print pcseparated
  file-print pcdivorced
  file-print pcwidowed
  file-print " "
  file-print school_age
  file-print working_age
  file-print retired
  file-print " "
  file-close
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; plotting section
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

to update-plot
  set-current-plot "population"
  plot pop
  
  set-current-plot "Population Makeup"
  set-current-plot-pen "pcsingle"
  plot pcsingle
  set-current-plot-pen "pcmarried"
  plot pcmarried
  set-current-plot-pen "pccohab"
  plot pccohab
  set-current-plot-pen "pcseparated"
  plot pcseparated
  set-current-plot-pen "pcdivorced"
  plot pcdivorced  
  set-current-plot-pen "pcwidowed"
  plot pcwidowed
  
  set-current-plot "Age Groups"
  set-current-plot-pen "school_age"
  plot school_age
  set-current-plot-pen "working_age"
  plot working_age
  set-current-plot-pen "retired"
  plot retired
end
@#$#@#$#@
GRAPHICS-WINDOW
48
563
459
995
-1
-1
1.0
0
10
1
1
1
0
0
0
1
0
400
0
400
1
1
1
ticks

CC-WINDOW
5
1009
1019
1104
Command Center
0

BUTTON
10
10
162
56
Load Initial Population
setup
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL

BUTTON
10
65
158
111
Run Simulation
go
T
1
T
OBSERVER
NIL
NIL
NIL
NIL

PLOT
259
363
682
524
Population
Years
Individuals
0.0
10.0
0.0
10.0
true
false
PENS
"default" 1.0 0 -16777216 true

INPUTBOX
10
125
113
185
Households
5000
1
0
Number

INPUTBOX
10
195
113
255
stop_after
1
1
0
Number

PLOT
260
186
682
360
Population Makeup
Years
%
0.0
10.0
0.0
10.0
true
true
PENS
"pcmarried" 1.0 0 -16777216 true
"pcsingle" 1.0 0 -13791810 true
"pccohab" 1.0 0 -1184463 true
"pcseparated" 1.0 0 -10899396 true
"pcwidowed" 1.0 0 -7500403 true
"pcdivorced" 1.0 0 -955883 true

MONITOR
690
190
765
235
Married
pcmarried
1
1
11

MONITOR
775
190
850
235
Single
pcsingle
1
1
11

MONITOR
690
240
765
285
Cohabiting
pccohab
1
1
11

MONITOR
775
240
850
285
Separated
pcseparated
1
1
11

MONITOR
690
290
765
335
Widowed
pcwidowed
1
1
11

MONITOR
775
290
850
335
Divorced
pcdivorced
1
1
11

SLIDER
10
265
190
298
propensity_to_marry
propensity_to_marry
0.5
2
1
0.01
1
NIL
HORIZONTAL

SLIDER
10
305
189
338
propensity_to_divorce
propensity_to_divorce
0.5
2
1
0.01
1
NIL
HORIZONTAL

SLIDER
10
345
189
378
propensity_to_cohabit
propensity_to_cohabit
0.5
2
1
0.01
1
NIL
HORIZONTAL

SLIDER
10
385
190
418
propensity_to_split
propensity_to_split
0.5
2
1
0.01
1
NIL
HORIZONTAL

SLIDER
10
425
190
458
birth_rate
birth_rate
0.5
2
1
0.01
1
NIL
HORIZONTAL

SLIDER
10
465
190
498
mortality_rate
mortality_rate
0.5
2
1
0.01
1
NIL
HORIZONTAL

PLOT
260
10
682
182
Age Groups
Years
%
0.0
10.0
0.0
10.0
true
true
PENS
"school_age" 1.0 0 -955883 true
"working_age" 1.0 0 -13840069 true
"retired" 1.0 0 -8630108 true

MONITOR
165
10
253
59
NIL
Current_Year
0
1
12

MONITOR
694
372
765
417
Population
pop
0
1
11

MONITOR
694
423
764
468
Households
houses
0
1
11

MONITOR
690
10
765
55
School Age
school_age
1
1
11

MONITOR
690
61
765
106
Working Age
working_age
1
1
11

MONITOR
690
111
765
156
Retired
retired
1
1
11

TEXTBOX
120
135
230
177
Number of households\nto read from file
11
125.0
1

TEXTBOX
120
210
215
238
Number of years\nto simulate
11
125.0
1

TEXTBOX
770
15
830
33
(Under 18)
11
25.0
1

TEXTBOX
770
65
820
91
(18 to 64)
11
65.0
1

TEXTBOX
770
115
840
141
(65 and over)
11
125.0
0

TEXTBOX
860
13
1010
32
Tyche Version 1.1
15
126.0
1

@#$#@#$#@
WHAT IS IT?
-----------
A microsimulation of the UK population.

An initial population of individuals and households is read in from a file then advanced over time to simulate how the population would change as people age. The annual probability of events such as birth and death as well as partnership formation and dissolution are used to predict the path of each inidvidual. The results from
this are then aggregated to forecast the makeup of the future population.

It is based on actual data from the British Household Panel Survey.


HOW TO USE IT
-------------
The model can be run by clicking on 'Load Initial Population'. This reads in a representative sample of the UK population as it was in 1991.

When this is complete, clicking on the 'Run Simulation' button will begin advancing the population over time.

The number of households to load in can be set in the 'Households' input box. Loading the maximum number of households gives more accurate results but makes the model run more slowly.

The world grid should be set to have just enough patches to accommodate the population. An excessive number of patches will slow the model.

Setting the 'stop_after' value tells the model to halt after a certain number of years.

The sliders allow alterations to be made to current probabilities of events. Moving it
to the left reduces the probability by up to 50%. Moving it to the right edge would double the probability of the event.

 
THINGS TO NOTICE
----------------
Demographic changes tend to happen fairly slowly. By the late 1990s it becomes apparant that the proportion of the population who are cohabiting is increasing and the proportion who are married is decreasing. Also, the proportion aged over 65 is increasing while the proportion aged under 18 is decreasing.


THINGS TO TRY
-------------
Run the model for a hundred years or more to see what happens to the ageing population.

Adjust the birth rate, mortality rate and the other sliders to see what effect this has on the population.


HOW IT WORKS
------------
The annual transition probabilities for the various events are implemented in a series of modules.

After the initial population is loaded, for each year and for each individual, the modules are run in the following order -

mortality
fertility
leaving home
marriage dissolution
cohabitation dissolution
cohabitation formation
marriage
age
return home after dissolution
return home
children follow parents


How these work is described in more detail below.

Loading data

The household file is read first. This sets up patches ready to be occuppied by the agents. Next the agent file is read. Each agent moves to the house indicated by the casenumber.

Children are assigned to a parent they will be dependent upon in the event of the couple separating. In 9 out of 10 cases, following the SAGE model, this is the female. In the birth module, all offspring depend on their mother.


Mortality

The annual probability of death was taken from the Office for National Statistics (ONS) Life Tables.

For those aged under 40, this was approximated by a simple average. For those over 40, this was fitted with an exponential function.

To test whether the agent dies before the end of the year
	for males aged 40 and younger the probability of 
	not surviving p = 0.000736
	for males over 40 p = 0.00003 e^0.0954 age

	for females 40 and younger p = 0.000411
	for females over 40 p = 0.00001 e^0.1017 age

	
Birth

agents must be aged from 15 to 60
probability of a birth in any year is calculated using three logistic equations
derived from a pooled sample from the BHPS. Pairs of years from waves J 		to O are combined.
Separate logistic equations are obtained. One for married women, one 			for cohabitors and one for singles and others. They are all based on 			age, number of children and age of the youngest child.
Probabilities of twins or triplets are modelled using data from SAGE
Technical Note no. 4.
When a birth is to an individual not married or cohabiting but living with another, they move to a new location with a nominal 0.5 probability. This is meant to reflect the formation of households with a single parent. The model seems insensitive to the precise value of this parameter.


Leaving home

Individuals who are not living alone, are aged 18 and over, are single and have no dependents move to an unoccupied patch with a probability of 0.1


Marriage dissolution

Separete logistic models for males and females determine which marriages end in the current year. The model that operates for the current individual is alternately male
of female dominated with a probability of 0.5. If an agent is selected for separation, their status is set to separated, and a newlyseparated flag is set. It will be reset
at the start of the next year's cycle to prevent newly separated individuals from
re-marrying in the current year. Next, the status of their partner is set to separated. With a probability of 0.9, the male moves to a vacant patch.


Cohabitation dissolution

This module operates in the same way as the marriage dissolution module although the actual probabilities are slightly different.


Cohabitation formation

Separate probabilities are calculated for males and females with age and current status taken into account. In the female model, birth in the current year's fertility module
is also allowed for. If selected, the first step is to check whether there is anyone 
of appropriate age available in the population. If not the process is aborted and a warning message appears in the output window. If there is one, the turtle's status is set to cohabiting and it moves to a vacant patch. Next a search is made for a partner with an age difference with a probability derived from SAGE documentation. If there is one, its status is set to cohabiting and it moves to the patch of the current turtle. If not, a wider age range is considered.


Marriage formation

Couples who are already cohabiting are considered for marriage provided the cohabitation did not start in the current year. This is determined by the newcohabitation flag. If selected, the resident couple set their status to married. They do not move house.

Individuals living separately are considered for marriage in a way similar to the cohabitation module.


Age is incremented by 1 year ready for the next cycle


Returning home after separation

Individuals whoose cohabitation or marriage desolved in the current year return to their parents or original home if they are aged under 30 and have no dependents.


Returning home for all agents

With a probability of 0.01, individuals aged under 30 who are not married or cohabiting and living alone or with one other return to their original address. In many cases, they will already be at this address. This is meant to reflect the movement of younger individuals sharing accommodation for example.


Locating children

Offspring record the who number of the parent it is designated as dependent upon. At the end of the year, all offspring check the location of this parent and move to their current location.


CREDITS AND REFERENCES
----------------------
Developed by Tony Lawson as part of a PhD project at the University of Essex
to develop a microsimulation of household demand. Sponsored by the ESRC and BT.
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

circle
false
0
Circle -7500403 true true 0 0 300

circle 2
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

cylinder
false
0
Circle -7500403 true true 0 0 300

dot
false
0
Circle -7500403 true true 90 90 120

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

line half
true
0
Line -7500403 true 150 0 150 150

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

square
false
0
Rectangle -7500403 true true 30 30 270 270

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 4.0.2
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
default
0.0
-0.2 0 0.0 1.0
0.0 1 1.0 0.0
0.2 0 0.0 1.0
link direction
true
0
Line -7500403 true 150 150 90 180
Line -7500403 true 150 150 210 180

@#$#@#$#@
